#!/usr/bin/env python

__author__ = 'Jeffrey Knockel'
__copyright__ = 'Copyright 2013 Jeffrey Knockel'
__license__ = 'GPL v2'

import sys

from dpkt import Packet
from dpkt.tcp import TCP, TH_RST, TH_ACK

from nslookup4 import lookup_or_die
from sequence import TCPSequence
from sniffing import fragmentbuf, tcp_flags_to_string, start_sniffing4 as start_sniffing

def worker(sniffer, address, port, mtu):
    seq = TCPSequence(address, iface=sniffer.iface)
    timeout = sniffer.timeout()
    while True:
        timeout.add_time(1.0)
        request = seq.next_packet_testing_mtu(mtu, dport=port)
        send_tcp(sniffer, request)
        received = False
        while True:
            p = receive_tcp(sniffer, request, timeout)
            if p is None:
                if not received:
                    print '*'
                break
            else:
                received = True
                print '%s: %d' % (tcp_flags_to_string(p.data.flags), p.id)
        sys.stdout.flush()

def send_tcp(sniffer, ps):
    if isinstance(ps, Packet):
        ps = [ps]
    for p in ps:
        fragmented = fragmentbuf(p)
        sniffer.sendbuf(fragmented, p.dst)

def receive_tcp(sniffer, sents, timeout):
    if isinstance(sents, Packet):
        sents = [sents]
    while True:
        p = sniffer.sniff(timeout)
        if p is None:
            return None
        if type(p.data) is TCP:
            p_tcp = p.data
            for sent in sents:
                sent_tcp = sent.data
                if p.src == sent.dst and p_tcp.sport == sent_tcp.dport and p_tcp.dport == sent_tcp.sport and ((p_tcp.flags & TH_ACK and p_tcp.ack - sent_tcp.seq & 0xffffffff == 1) or (not p_tcp.flags & TH_ACK and p_tcp.flags & TH_RST)):
                    return p

def usage(code=1):
    sys.stderr.write('Usage: %s HOST PORT MTU\n' % sys.argv[0])
    exit(code)

def main():
    try:
        if len(sys.argv) != 4:
            usage()
        host = sys.argv[1]
        try:
            port = int(sys.argv[2])
            mtu = int(sys.argv[3])
        except ValueError:
            usage()
        addr = lookup_or_die(host)[0]
        start_sniffing(worker, addr, port, mtu)
    except KeyboardInterrupt:
        sys.stderr.write('Received SIGINT\n')

if __name__ == '__main__':
    main()
